#define SENSOR_PIN A0
#define ALPHA 0.75
#define LED_PIN 9
#define BEAT_BUFFER_SIZE 7
#define THRESHOLD 3
#define MIN_VALUE 100

void setup() {
  pinMode(LED_PIN, OUTPUT);
}

void loop() {
  float value = readSensor();

  if (detectBeat(value)) {
    digitalWrite(LED_PIN, HIGH);
  } else {
    digitalWrite(LED_PIN, LOW);
  }
}

bool detectBeat(float currentValue) {
  static float buffer[BEAT_BUFFER_SIZE] = { 0 };
  static int index = 0;
  static int counter = 0;
  static bool isRising = false;
  bool detected = false;

  if (currentValue < MIN_VALUE) {
    return false;
  }

  buffer[index] = currentValue;
  index = (index + 1) % BEAT_BUFFER_SIZE;

  int half = BEAT_BUFFER_SIZE / 2;
  float olderSum = 0;
  float newerSum = 0;
  int olderStart = index;
  int newerStart = olderStart + BEAT_BUFFER_SIZE - half;
  
  for (int i = 0; i < half; i++) {
    olderSum += buffer[(olderStart + i) % BEAT_BUFFER_SIZE];
    newerSum += buffer[(newerStart + i) % BEAT_BUFFER_SIZE];
  }

  counter += newerSum > olderSum ? 1 : -1;

  if (abs(counter) > THRESHOLD) {
    detected = counter > 0 && !isRising;
    isRising = counter > 0;
    counter = 0;
  }

  return detected;
}

float readSensor() {
  static float last = 0;
  float sum = 0;

  for (int i = 0; i < 20; i++) {
    sum += analogRead(SENSOR_PIN);
    delay(1);
  };

  last = ALPHA * (sum / 20) + (1 - ALPHA) * last;
  return last;
}
